#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2018 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import requests
import html
from pandas import DataFrame

from .utils import (
    MG_SEQ_URL,
    MG_SAMPLE_URL,
    MG_RUN_URL,
)


logger = logging.getLogger(__name__)


def sequence_search(args):

    """
    Process given fasta file
    """
    args = vars(args)
    for s in args.pop("sequence"):
        with open(s) as f:
            sequence = f.read()
            seq = SequenceSearch(
                sequence,
                database=args.pop("database", "full"),
                seq_evalue_threshold=args.pop(
                    "seq_evalue_threshold", None),
                hit_evalue_threshold=args.pop(
                    "hit_evalue_threshold", None),
                report_seq_evalue_threshold=args.pop(
                    "report_seq_evalue_threshold", None),
                report_hit_evalue_threshold=args.pop(
                    "report_hit_evalue_threshold", None),
                seq_bitscore_threshold=args.pop(
                    "seq_bitscore_threshold", None),
                hit_bitscore_threshold=args.pop(
                    "hit_bitscore_threshold", None),
                report_seq_bitscore_threshold=args.pop(
                    "report_seq_bitscore_threshold", None),
                report_hit_bitscore_threshold=args.pop(
                    "report_hit_bitscore_threshold", None),
            )
            results = seq.analyse_sequence()
            job_uuid = results['results']['uuid']
            logger.debug("Job %s" % job_uuid)
            seq.save_to_csv(seq.fetch_results(results), filename=job_uuid)


class SequenceSearch(object):

    """
    Helper tool allowing to search non-redundant protein database using HMMER
    and fetch environmental metadata.
    """

    sequence = None
    database = "full"

    def __init__(self, sequence, database="full", *args, **kwargs):
        self.sequence = sequence
        self.database = database
        self.seq_evalue_threshold = kwargs.pop(
            "seq_evalue_threshold", None)
        self.hit_evalue_threshold = kwargs.pop(
            "hit_evalue_threshold", None)
        self.report_seq_evalue_threshold = kwargs.pop(
            "report_seq_evalue_threshold", None)
        self.report_hit_evalue_threshold = kwargs.pop(
            "report_hit_evalue_threshold", None)
        self.seq_bitscore_threshold = kwargs.pop(
            "seq_bitscore_threshold", None)
        self.hit_bitscore_threshold = kwargs.pop(
            "hit_bitscore_threshold", None)
        self.report_seq_bitscore_threshold = kwargs.pop(
            "report_seq_bitscore_threshold", None)
        self.report_hit_bitscore_threshold = kwargs.pop(
            "report_hit_bitscore_threshold", None)

    def analyse_sequence(self):
        data = {
            "seqdb": self.database,
            "seq": self.sequence,
        }
        if self.seq_evalue_threshold is not None:
            data["incE"] = self.seq_evalue_threshold
        if self.hit_evalue_threshold is not None:
            data["incdomE"] = self.hit_evalue_threshold
        if self.report_seq_evalue_threshold is not None:
            data["E"] = self.report_seq_evalue_threshold
        if self.report_hit_evalue_threshold is not None:
            data["domE"] = self.report_hit_evalue_threshold
        if self.seq_bitscore_threshold is not None:
            data["incT"] = self.seq_bitscore_threshold
        if self.hit_bitscore_threshold is not None:
            data["incdomT"] = self.hit_bitscore_threshold
        if self.report_seq_bitscore_threshold is not None:
            data["T"] = self.report_seq_bitscore_threshold
        if self.report_hit_bitscore_threshold is not None:
            data["domT"] = self.report_hit_bitscore_threshold

        headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/x-www-form-urlencoded',
        }
        logger.debug("POST: %r" % data)
        return requests.post(MG_SEQ_URL, data=data, headers=headers).json()

    def make_request(self, accession):
        if accession is None:
            return None
        headers = {
            'Accept': 'application/vnd.api+json',
        }
        r = requests.get(
            MG_SAMPLE_URL.format(**{'accession': accession}),
            headers=headers
        )
        if r.status_code != requests.codes.ok:
            r = requests.get(
                MG_RUN_URL.format(**{'accession': accession}),
                headers=headers
            )
        return r.json()

    def get_sample_metadata(self, accession, request):
        _meta = {}
        try:
            metadata = request['data']['attributes']['sample-metadata']
            logger.debug(metadata)
        except KeyError:
            try:
                metadata = \
                    request['included'][0]['attributes']['sample-metadata']
                logger.debug(metadata)
            except KeyError:
                return _meta
        for m in metadata:
            unit = html.unescape(m['unit']) if m['unit'] else ""
            _meta[m['key'].replace(" ", "_")] = "{value} {unit}".format(value=m['value'], unit=unit)  # noqa

        try:
            _meta['biome'], _meta['lineage'] = self.get_biome(request)
        except ValueError:
            pass
        return _meta

    def get_biome(self, request):
        _biome = None
        try:
            _biome = request['data']['relationships']['biome']['data']['id']
            logger.debug(_biome)
        except KeyError:
            try:
                _biome = request['included'][0]['relationships']['biome']['data']['id']  # noqa
                logger.debug(_biome)
            except KeyError:
                pass
        if _biome is not None:
            return _biome.split(":")[-1], _biome
        raise ValueError("Biome doesn't exist.")

    def fetch_results(self, results):
        csv_rows = dict()
        for h in results['results']['hits']:
            acc2 = h.get('acc2', None)
            if acc2 is not None:
                for accession in acc2.split(","):
                    accession = accession.strip("...")
                    logger.debug("Accession %s" % accession)
                    uuid = "{n} {a}".format(
                        **{'n': h['name'], 'a': accession})
                    csv_rows[uuid] = dict()
                    csv_rows[uuid]['accessions'] = accession
                    csv_rows[uuid]['kg'] = h.get('kg', '')
                    csv_rows[uuid]['taxid'] = h.get('taxid', '')
                    csv_rows[uuid]['name'] = h.get('name', '')
                    csv_rows[uuid]['desc'] = h.get('desc', '')
                    csv_rows[uuid]['pvalue'] = h.get('pvalue', '')
                    csv_rows[uuid]['species'] = h.get('species', '')
                    csv_rows[uuid]['score'] = h.get('score', '')
                    csv_rows[uuid]['evalue'] = h.get('evalue', '')
                    csv_rows[uuid]['nreported'] = h.get('nreported', '')
                    csv_rows[uuid]['uniprot'] = ",".join(
                        [i[0] for i in h.get('uniprot_link', [])])

                    req = self.make_request(accession)
                    _meta = self.get_sample_metadata(
                        accession=accession, request=req
                    )
                    csv_rows[uuid].update(_meta)
        return csv_rows

    def save_to_csv(self, csv_rows, filename):
        df = DataFrame(csv_rows).T
        df.index.name = 'name'
        filename = "{}_sequence_search.csv".format(filename)
        df.to_csv(filename)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2018 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import requests
import xmltodict
import json

try:
    from json.decoder import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError

from pandas import DataFrame

from .utils import (
    sample_url,
    metadata_url
)

logger = logging.getLogger(__name__)


def original_metadata(args):

    """
    Process given accessions
    """

    for accession in args.accession:
        logger.debug("Accession %s" % accession)
        om = OriginalMetadata(accession)
        om.save_to_csv(om.fetch_metadata())


class OriginalMetadata(object):

    """
    Helper tool allowing to download original metadata for the given accession.
    """

    accession = None

    def __init__(self, accession, *args, **kwargs):
        self.accession = accession

    def get_metadata(self, sample_accession):

        """
        Process XML.
        """

        meta = dict()
        sample = requests.get(
            metadata_url().format(**{'accession': sample_accession})
        )
        x = json.loads(json.dumps(xmltodict.parse(sample.content)))
        for m in x['ROOT']['SAMPLE']['SAMPLE_ATTRIBUTES']['SAMPLE_ATTRIBUTE']:
            try:
                key = m['TAG']
            except KeyError:
                continue
            try:
                value = m['VALUE']
            except KeyError:
                value = None
            meta[key] = value
        return meta

    def fetch_metadata(self):
        resp = requests.get(
            sample_url().format(**{'accession': self.accession})
        )
        try:
            resp = resp.json()
        except JSONDecodeError:
            return

        _accessions = {
            r['run_accession']:
                {
                    'sample_accession': r['secondary_sample_accession'],
                    'read_depth': r['depth']
                }
            for r in resp
        }

        meta_csv = dict()
        _sample = None
        _meta = None
        for (run, sample) in _accessions.items():
            if sample != _sample:
                _meta = self.get_metadata(sample['sample_accession'])
            meta_csv[run] = _meta
            meta_csv[run]['Sample'] = sample['sample_accession']
            meta_csv[run]['Read depth'] = sample['read_depth']
            _sample = sample['sample_accession']
        return meta_csv

    def save_to_csv(self, meta_csv, filename=None):
        df = DataFrame(meta_csv).T
        df.index.name = 'Run'
        if filename is None:
            filename = "{}.csv".format(self.accession)
        df.to_csv(filename)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2018 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

logger = logging.getLogger(__name__)


MG_SEQ_URL = 'https://www.ebi.ac.uk/metagenomics/sequence-search/search/phmmer'

MG_SAMPLE_URL = \
    'https://www.ebi.ac.uk/metagenomics/api/latest/samples/{accession}'

MG_RUN_URL = (
    'https://www.ebi.ac.uk/metagenomics/api/latest/runs/{accession}'
    '?include=sample'
)


def sample_url():
    return (
        'https://www.ebi.ac.uk/ena/portal/api/search?'
        'result=read_run'
        '&query=study_accession%3D{accession}%20'
        'OR%20secondary_study_accession%3D{accession}'
        '&fields=run_accession,secondary_sample_accession,sample_accession'
        ',depth'
        '&format=json'
    )


def metadata_url():
    return 'https://www.ebi.ac.uk/ena/data/view/{accession}&display=xml'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2018 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import logging
import argparse
import textwrap
import mg_toolkit

from .metadata import original_metadata  # noqa
from .search import sequence_search  # noqa

__version__ = '0.5.0'
__all__ = [
    'original_metadata',
    'sequence_search',
]


def is_file(filename):
    if not os.path.isfile(filename):
        msg = "{0} is not a directory".format(filename)
        raise argparse.ArgumentTypeError(msg)
    else:
        return filename


def main():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=textwrap.dedent('''\
            Metagenomics toolkit
            --------------------''')
    )
    parser.add_argument(
        '-V', '--version', action='version', version=__version__,
        help='print version information'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='print debugging information'
    )

    subparsers = parser.add_subparsers(dest='tool')

    original_metadata_parser = subparsers.add_parser(
        'original_metadata', help='Download original metadata.'
    )
    original_metadata_parser.add_argument(
        '-a', '--accession', required=True, nargs='+',
        help='Provide study accession, e.g. PRJEB1787 or ERP001736.'
    )

    sequence_search_parser = subparsers.add_parser(
        'sequence_search',
        help='Search non-redundant protein database using HMMER'
    )

    sequence_search_parser.add_argument(
        '-seq', '--sequence', required=True, type=is_file, nargs='+',
        help='Provide path to fasta file.'
    )
    sequence_search_parser.add_argument(
        '-db', '--database', type=str, choices=['full', 'all', 'partial', ],
        default='full',
        help='Choose peptide database (default: %(default)s).'
    )

    sequence_search_subparser = \
        sequence_search_parser.add_subparsers(dest='threshold')

    # e-value
    evalue_parser = sequence_search_subparser.add_parser(
        'evalue',
        help='Search non-redundant protein database using HMMER.'
    )
    evalue_parser.add_argument(
        '-incE', '--seq-evalue-threshold', type=float, default=0.01,
        help=('Sequence E-value threshold. Accepted value 0 < x ≤ 10 '
              '(default: %(default)s).')
    )
    evalue_parser.add_argument(
        '-incdomE', '--hit-evalue-threshold', type=float, default=0.03,
        help=('Hit E-value threshold. Accepted value 0 < x ≤ 10 '
              '(default: %(default)s).')
    )
    evalue_parser.add_argument(
        '-E', '--report-seq-evalue-threshold', type=float, default=1,
        help=('Sequence E-value threshold (reporting).'
              'Accepted value 0 < x ≤ 10 (default: %(default)s).')
    )
    evalue_parser.add_argument(
        '-domE', '--report-hit-evalue-threshold', type=float, default=1,
        help=('Hit E-value threshold (reporting). Accepted value 0 < x ≤ 10 '
              '(default: %(default)s).')
    )

    # bit score
    bitscore_parser = sequence_search_subparser.add_parser(
        'bitscore',
        help='Search non-redundant protein database using HMMER.'
    )
    bitscore_parser.add_argument(
        '-incT', '--seq-bitscore-threshold', type=float, default=25,
        help=('Sequence bit score threshold. Accepted values x > 0 '
              '(default: %(default)s).')
    )
    bitscore_parser.add_argument(
        '-incdomT', '--hit-bitscore-threshold', type=float, default=23,
        help=('Hit bit score threshold. Accepted values x > 0 '
              '(default: %(default)s).')
    )
    bitscore_parser.add_argument(
        '-T', '--report-seq-bitscore-threshold', type=float, default=7,
        help=('Sequence E-value threshold (reporting). Accepted values x > 0 '
              '(default: %(default)s).')
    )
    bitscore_parser.add_argument(
        '-domT', '--report-hit-bitscore-threshold', type=float, default=5,
        help=('Hit E-value threshold (reporting). Accepted values x > 0 '
              '(default: %(default)s).')
    )

    args = parser.parse_args()

    if args.debug:
        log_level = logging.DEBUG
    else:
        log_level = logging.WARN

    logging.basicConfig(format='%(levelname)s: %(message)s', level=log_level)

    if args.tool is not None:
        return getattr(mg_toolkit, args.tool)(args)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2018 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import os

from setuptools import setup, find_packages

_base = os.path.dirname(os.path.abspath(__file__))
_requirements = os.path.join(_base, 'requirements.txt')
_requirements_test = os.path.join(_base, 'requirements-test.txt')

install_requirements = []
with open(_requirements) as f:
    install_requirements = f.read().splitlines()

test_requirements = []
if "test" in sys.argv:
    with open(_requirements_test) as f:
        test_requirements = f.read().splitlines()

long_description = ""
with open(os.path.join(_base, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="mg-toolkit",
    url='https://github.com/EBI-metagenomics/emg-toolkit',
    author='Ola Tarkowska',
    author_email='olat@ebi.ac.uk',
    description='Metagenomics toolkit',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(exclude=['ez_setup']),
    version="0.5.0",
    install_requires=install_requirements,
    setup_requires=['pytest-runner'],
    tests_require=test_requirements,
    include_package_data=True,
    zip_safe=False,
    license='Apache Software License',
    classifiers=[
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
    test_suite="tests",
    entry_points={
        'console_scripts': [
            'mg-toolkit=mg_toolkit:main',
        ],
    },
)
